package cn.skcks.docking.wx.services.mp.service.app;

import cn.hutool.core.date.DateUtil;
import cn.skcks.docking.wx.common.json.JsonUtils;
import cn.skcks.docking.wx.common.page.PageWrapper;
import cn.skcks.docking.wx.orm.entity.MpApp;
import cn.skcks.docking.wx.orm.mapper.MpAppMapper;
import cn.skcks.docking.wx.services.mp.common.redis.WxRedisOps;
import cn.skcks.docking.wx.services.mp.service.kefu.MpKeFuService;
import cn.skcks.docking.wx.services.mp.service.message.handler.MessageHandlerService;
import cn.skcks.docking.wx.services.mp.vo.verify.SignatureVo;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import jakarta.annotation.PostConstruct;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import me.chanjar.weixin.mp.config.WxMpConfigStorage;
import me.chanjar.weixin.mp.config.impl.WxMpRedisConfigImpl;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ForkJoinPool;

@Slf4j
@Service
@RequiredArgsConstructor
public class MpAppService {
    private final MpAppMapper mpAppMapper;
    private final WxRedisOps wxRedisOps;
    private final MpKeFuService mpKeFuService;
    private final MessageHandlerService messageHandlerService;

    @Getter
    private final Map<String, WxMpService> serviceMap = new ConcurrentHashMap<>();

    @PostConstruct
    private void init(){
        LambdaQueryChainWrapper<MpApp> queryWrapper = new LambdaQueryChainWrapper<>(mpAppMapper);
        queryWrapper.ne(MpApp::getEnable, false);
        List<MpApp> mpApps = queryWrapper.list();

        new ForkJoinPool(Runtime.getRuntime().availableProcessors()).execute(() -> {
            mpApps.parallelStream().forEach(mpApp -> {
                WxMpRedisConfigImpl config = new WxMpRedisConfigImpl(wxRedisOps, "wx_mp");
                config.setAppId(mpApp.getAppId());
                config.setSecret(mpApp.getAppSecret());
                config.setToken(mpApp.getToken());
                config.setAesKey(mpApp.getEncodingAesKey());

                WxMpService service = new WxMpServiceImpl();
                service.setWxMpConfigStorage(config);

                serviceMap.put(mpApp.getAppId(), service);
            });
        });
    }

/**
 * 处理微信公众号消息请求的服务方法
 *
 * @param request   HTTP请求对象
 * @param response  HTTP响应对象
 * @param appId     应用ID
 * @param signatureVo 签名验证信息
 * @throws IOException 如果处理过程中发生IO异常
 */
public void service(HttpServletRequest request, HttpServletResponse response, String appId, SignatureVo signatureVo) throws IOException {
    // 设置响应内容类型和状态码
    response.setContentType("text/html;charset=utf-8");
    response.setStatus(HttpServletResponse.SC_OK);

    // 从服务映射中获取WxMpService实例
    WxMpService wxMpService = serviceMap.get(appId);
    if (ObjectUtils.anyNull(appId, signatureVo, wxMpService)) {
        // 如果参数为空，返回非法请求响应
        response.getWriter().println("非法请求");
        return;
    }

    // 验证签名是否合法
    if (!wxMpService.checkSignature(signatureVo.getTimestamp(), signatureVo.getNonce(), signatureVo.getSignature())) {
        // 如果签名不合法，返回非法请求响应
        response.getWriter().println("非法请求");
        return;
    }

    // 处理微信服务器的验证请求
    String echostr = request.getParameter("echostr");
    if (StringUtils.isNotBlank(echostr)) {
        // 返回echostr以通过微信服务器的验证
        response.getWriter().println(echostr);
        return;
    }

    // 获取加密类型，默认为"raw"
    String encryptType = StringUtils.isBlank(request.getParameter("encrypt_type")) ? "raw" : request.getParameter("encrypt_type");

    // 处理消息并获取解析后的WxMpXmlMessage对象
    WxMpXmlMessage inMessage = messageHandlerService.handleMessage(request, wxMpService, signatureVo);
    if (inMessage == null) {
        // 如果无法识别的加密类型，返回不可识别的消息响应
        response.getWriter().println("不可识别的加密类型");
    } else {
        // 记录接收到的消息内容
        log.debug("\n[↓] 接收消息: \n{}", inMessage);

        // 构建自动回复的消息
        WxMpXmlOutMessage outMessage = WxMpXmlOutMessage.TEXT()
                .toUser(inMessage.getFromUser())
                .fromUser(inMessage.getToUser())
                .content(DateUtil.now())
                .build();
        log.debug("\n[↑] 响应消息: \n{}", JsonUtils.toCompressJson(outMessage));

        // 根据加密类型返回响应
        if (StringUtils.equalsIgnoreCase(encryptType, "raw")) {
            response.getWriter().write(outMessage.toXml());
        } else {
            response.getWriter().write(outMessage.toEncryptedXml(wxMpService.getWxMpConfigStorage()));
        }
    }
}


    /**
     * 获取所有公众号列表
     *
     * @author shikong
     * @since 2024-12-07
     */
    public List<MpApp> getMpAppList(){
        return mpAppMapper.selectList(null);
    }


    /**
     * 分页查询公众号列表
     *
     * @author shikong
     * @since 2024-12-07
     */
    public PageWrapper<MpApp> getMpAppPageList(Integer pageNum, Integer pageSize){
        Page<MpApp> mpAppPage = mpAppMapper.selectPage(new Page<>(pageNum, pageSize), null);
        return PageWrapper.of(mpAppPage);
    }
}
